#!/usr/bin/env perl

#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# This file is part of G-language Genome Analysis Environment package
#
#     Copyright (C) 2001-2007 Keio University
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# 
#   $Id: G.pm,v 1.4 2002/07/30 17:40:56 gaou Exp $
#
# G-language GAE is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
# 
# G-language GAE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public
# License along with G-language GAE -- see the file COPYING.
# If not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# 
#END_HEADER
#
# written by Kazuharu Arakawa <gaou@g-language.org> at
# G-language Project, Institute for Advanced Biosciences, Keio University.
#

package G;

use strict;

# import "prelude" extensions
use SubOpt;
use Rcmd;

# import messenger
use G::Messenger;

# import "skyline" extensions
require G::Skyline;
require G::DynamicLoader;
use G::Inspire;

# import db classes
use G::DB::SDB;
use G::DB::GDBI;

# import "odyssey" classes
use G::Seq::AminoAcid;
use G::Seq::Codon;
use G::Seq::Consensus;
use G::Seq::Eliminate;
use G::Seq::FreeEnergy;
use G::Seq::GCskew;
use G::Seq::Operon;
use G::Seq::OverLapping;
use G::Seq::Primitive;
use G::Seq::PatSearch;
use G::Seq::Tandem;
use G::Seq::Util;
use G::Seq::Markov;
use G::Seq::ORF;
use G::Seq::Align;
use G::Seq::Usage;
use G::Seq::COMGA;
use G::Seq::ImaGene;
use G::Tools::Graph;
use G::Tools::H2v;
use G::Tools::Mapping;
use G::Tools::Blast;
use G::Tools::Fasta;
use G::Tools::Alignment;
use G::Tools::HMMER;
use G::Tools::Repeat;
use G::Tools::EPCR;
use G::Tools::SIM4;
use G::Tools::PBS;
use G::Tools::Cap3;
use G::Tools::COGs;
use G::Tools::GPAC;
use G::Tools::GOA;
use G::Tools::GlimmerM;
use G::Tools::Literature;
use G::Tools::KEGG_API;
use G::Tools::KEGG_API3;
use G::Tools::RCluster;
use G::Tools::PEC;
use G::SystemsBiology::Serizawa;
use G::SystemsBiology::Pathway;
use G::SystemsBiology::PathwayAlignment;
use G::SystemsBiology::EcellReader;
use G::SystemsBiology::KEGG;
use G::SystemsBiology::BioLayout;
use G::SystemsBiology::Interaction;
use G::SystemsBiology::DotE;

# import "presage" classes
use G::System::BAS;
use G::System::GEMS;
use G::System::COMGA;
use G::System::STeP;
use G::System::CHI;
use G::System::ReL8;
use G::System::FuncD;

use vars qw($VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(G::Skyline Exporter);

# export odyssay functions
push(@EXPORT, @G::Seq::AminoAcid::EXPORT);
push(@EXPORT, @G::Seq::Codon::EXPORT);
push(@EXPORT, @G::Seq::Consensus::EXPORT);
push(@EXPORT, @G::Seq::Eliminate::EXPORT);
push(@EXPORT, @G::Seq::FreeEnergy::EXPORT);
push(@EXPORT, @G::Seq::GCskew::EXPORT);
push(@EXPORT, @G::Seq::Operon::EXPORT);
push(@EXPORT, @G::Seq::OverLapping::EXPORT);
push(@EXPORT, @G::Seq::Primitive::EXPORT);
push(@EXPORT, @G::Seq::PatSearch::EXPORT);
push(@EXPORT, @G::Seq::Tandem::EXPORT);
push(@EXPORT, @G::Seq::Util::EXPORT);
push(@EXPORT, @G::Seq::Markov::EXPORT);
push(@EXPORT, @G::Seq::ORF::EXPORT);
push(@EXPORT, @G::Seq::Align::EXPORT);
push(@EXPORT, @G::Seq::Usage::EXPORT);
push(@EXPORT, @G::Seq::COMGA::EXPORT);
push(@EXPORT, @G::Seq::ImaGene::EXPORT);
push(@EXPORT, @G::Tools::Graph::EXPORT);
push(@EXPORT, @G::Tools::H2v::EXPORT);
push(@EXPORT, @G::Tools::Mapping::EXPORT);
push(@EXPORT, @G::Tools::Blast::EXPORT);
push(@EXPORT, @G::Tools::Fasta::EXPORT);
push(@EXPORT, @G::Tools::Alignment::EXPORT);
push(@EXPORT, @G::Tools::HMMER::EXPORT);
push(@EXPORT, @G::Tools::Repeat::EXPORT);
push(@EXPORT, @G::Tools::EPCR::EXPORT);
push(@EXPORT, @G::Tools::SIM4::EXPORT);
push(@EXPORT, @G::Tools::PBS::EXPORT);
push(@EXPORT, @G::Tools::Cap3::EXPORT);
push(@EXPORT, @G::Tools::COGs::EXPORT);
push(@EXPORT, @G::Tools::GPAC::EXPORT);
push(@EXPORT, @G::Tools::GOA::EXPORT);
push(@EXPORT, @G::Tools::GlimmerM::EXPORT);
push(@EXPORT, @G::Tools::Literature::EXPORT);
push(@EXPORT, @G::Tools::KEGG_API::EXPORT);
push(@EXPORT, @G::Tools::KEGG_API3::EXPORT);
push(@EXPORT, @G::Tools::RCluster::EXPORT);
push(@EXPORT, @G::Tools::PEC::EXPORT);
push(@EXPORT, @G::SystemsBiology::Serizawa::EXPORT);
push(@EXPORT, @G::SystemsBiology::Pathway::EXPORT);
push(@EXPORT, @G::SystemsBiology::PathwayAlignment::EXPORT);
push(@EXPORT, @G::SystemsBiology::EcellReader::EXPORT);
push(@EXPORT, @G::SystemsBiology::KEGG::EXPORT);
push(@EXPORT, @G::SystemsBiology::BioLayout::EXPORT);
push(@EXPORT, @G::SystemsBiology::Interaction::EXPORT);
push(@EXPORT, @G::SystemsBiology::DotE::EXPORT);
push(@EXPORT, @G::System::BAS::EXPORT);
push(@EXPORT, @G::System::GEMS::EXPORT);
push(@EXPORT, @G::System::COMGA::EXPORT);
push(@EXPORT, @G::System::STeP::EXPORT);
push(@EXPORT, @G::System::CHI::EXPORT);
push(@EXPORT, @G::System::ReL8::EXPORT);
push(@EXPORT, @G::System::FuncD::EXPORT);
push(@EXPORT, @G::DB::SDB::EXPORT);
push(@EXPORT, @SubOpt::EXPORT);
push(@EXPORT, @G::Messenger::EXPORT);

# export plugin functions (overrides all)
push(@EXPORT, @G::DynamicLoader::EXPORT);


$VERSION = $G::Skyline::VERSION;

mkdir($ENV{HOME} . '/.glang/data/');


#::::::::::::::::::::::::::::::
#          Perldoc
#::::::::::::::::::::::::::::::

1;

__END__

=head1 NAME

G - G-language Genome Analysis Environment core interface module

=head1 SYNOPSIS

 use G;                          # Imports G-language GAE module 
   
 $gb = new G("ecoli.gbk");       # Creates G's instance as $gb 
                                 # At the same time, read in ecoli.gbk. 
                                 # Read the annotation and sequence 
                                 # information 
                                 # See DESCRIPTION for details
   
 $gb->seq_info();                # Prints the basic sequence information.

 $find_ori_ter($gb);             # Give $gb as the first argument to 
                                 # most of the analysis functions


=head1 DESCRIPTION

 The G-language GAE fully supports most sequence databases.

 Stored annotation information:

 LOCUS  
         $gb->{LOCUS}->{id}              -accession number 
         $gb->{LOCUS}->{length}          -length of sequence  
         $gb->{LOCUS}->{nucleotide}      -type of sequence ex. DNA, RNA  
         $gb->{LOCUS}->{circular}        -1 when the genome is circular.
                                          otherwise 0
         $gb->{LOCUS}->{type}            -type of species ex. BCT, CON  
         $gb->{LOCUS}->{date}            -date of accession 

 HEADER  
    $gb->{HEADER}  

 COMMENT  
    $gb->{COMMENT}  

 FEATURE  
         Each FEATURE is numbered(FEATURE1 .. FEATURE1172), and is a 
         hash structure that contains all the keys of Genbank.   
         In other words,  in most cases, FEATURE$i's hash at least 
         contains informations listed below: 
         $gb->{FEATURE$i}->{start}  
         $gb->{FEATURE$i}->{end}  
         $gb->{FEATURE$i}->{direction}
         $gb->{FEATURE$i}->{join}
         $gb->{FEATURE$i}->{note}  
         $gb->{FEATURE$i}->{type}        -CDS,gene,RNA,etc.

         To analyze each FEATURE, write: 

         foreach my $feature ($gb->feature()){
               print $gb->{$feature}->{type}, "\n";
         }  

         Each CDS is stored in a similar manner.
         There are 
         $gb->{CDS$i}->{start}
         $gb->{CDS$i}->{end}
         $gb->{CDS$i}->{direction}
         $gb->{CDS$i}->{join}
         $gb->{CDS$i}->{feature}         -number $n for $gb->{FEATURE$n}
                                          where "CDS$i" = "FEATURE$n"

         In the same manner, to analyze all CDS, write:  
 
         foreach my $cds ($gb->cds()){
               print $gb->{$cds}->{gene}, "\n";
         }

 BASE COUNT  
         $gb->{BASE_COUNT}  

 SEQ  
         $gb->{SEQ}              -sequence data following "ORIGIN" 


=head1 Supported methods of G-language Genome Analysis Environment

=cut

=head2 $gb = new G("genome file")

         Creates a G instance.
         First option is the filename of the database. Default format is
         the GenBank database.
         Second option specifies detailed actions.

           'no msg'                  suprresses all STDOUT messages printed 
                                     when loading a database, including the
                                     copyright info and sequence statistics.

           'no cache'                suppresses the use of database caching.
                                     By default, databases are cached for
                                     optimized performance. (since v.1.6.4)

           'force cache'             rebuilds database cache.

           'multiple locus'          this option merges multiple loci in the 
                                     database and load the information
                                     as G-language instance.

           'bioperl'                 this option creates a G instance from 
                                     a bioperl object. 
                                     eg. $bp = $bp->next_seq();       # bioperl
                                         $gb = new G($bp, "bioperl"); # G

           'longest ORF annotation'  this option predicts genes with longest ORF
                                     algorithm (longest frame from start codon
                                     to stop codon, with more than 17 amino 
                                     acids) and annotates the sequence.

           'glimmer annotation'      this option predicts genes using glimmer2,
                                     a gene prediction software for microbial
                                     genomes available from TIGR.
                                     http://www.tigr.org/softlab/
                                     Local installation of glimmer2 and setting
                                     of PATH environment value is required.

               - following options require bioperl installation -

           'Fasta'              this option loads a Fasta format database.
           'EMBL'               this option loads a EMBL  format database.
           'swiss'              this option loads a swiss format database.
           'SCF'                this option loads a SCF   format database.
           'PIR'                this option loads a PIR   format database.
           'GCG'                this option loads a GCG   format database.
           'raw'                this option loads a raw   format database.
           'ace'                this option loads a ace   format database.
           'net GenBank'        this option loads a GenBank format database from 
                                NCBI database. With this option, the first value to 
                                pass to new() function will be the accession 
                                number of the database.

=cut

=head2 $gb->output()

         Given a filename and an option, outputs the G-language data object 
         to the specified file in a flat-file database of a given format.
         The options are the same as those of new().  Default format is 'GenBank'.

         eg. $gb->output("my_genome.embl", "EMBL");
             $gb->output("my_genome.gbk"); # with GenBank you can ommit the option.

=cut

=head2 complement

         Given a sequence, returns its complement.

         eg. complement('atgc');  # returns 'gcat'

=cut

=head2 translate

         Given a sequence, returns its translated sequence.
         Regular codon table is used.
         eg. translate('ctggtg');  # returns 'LV'

=cut

=head2 $gb->seq()

         Returns the entire sequence. Same as $gb->{SEQ};

=cut

=head2 $gb->seq_info()

         Prints the basic information of the genome to STDOUT.

=cut

=head2 $gb->getseq()

         Given the start and end positions (starting from 0 as in Perl),
         returns the sequence specified.

         eg. $gb->getseq(1,3); # returns the 2nd, 3rd, and 4th nucleotides.

=cut

=head2 $gb->get_gbkseq()

         Given the start and end positions (starting from 1 as in 
         Genbank), returns the sequence specified.

         eg. $gb->get_gbkseq(1,3); # returns the 1st, 2nd, and 3rd nucleotides.

=cut

=head2 $gb->get_cdsseq()

         Given a CDS ID, returns the CDS sequence. 
         'complement' is properly parsed.

         eg. $gb->get_cdsseq('CDS1'); # returns the 'CDS1' sequence.

=cut

=head2 $gb->get_geneseq()

         Given a CDS ID, returns the CDS sequence, or the exon sequence
         If introns are present.
         'complement' is properly parsed, and introns are spliced out.

         eg. $gb->get_geneseq('CDS1'); # returns the 'CDS1' sequence or exon.

=cut

=head2 $gb->feature()

         Returns the array of all feature IDs.
         Features are ignored when $gb->{$feature}->{on} is 0.

         eg.
           foreach ($gb->feature()){
               $gb->get_cdsseq($_);
           }
           #prints all feature sequences.

         Optionally, feature type can be supplied to return only the
         specifies features.

         eg. $gb->feature("tRNA"); # returns feature IDs only for tRNAs

         Option of "all" always returns all features regardless of the
         value of $gb->{$feature}->{on}.

=cut

=head2 $gb->cds()

         Returns the array of all feature IDs of CDS.
         Features are ignored when $gb->{FEATURE$i}->{on} OR
         $gb->{CDS$i}->{on} is 0.

         !CAUTION! the object name is actually the FEATURE ID,
         to enable access to all feature values. However, most of the
         time you do not need to be aware of this difference.

         eg.
           foreach ($gb->cds()){
               $gb->get_geneseq($_);
           }
           #prints all gene sequences.

         Option of "all" always returns all features regardless of the
         value of $gb->{$feature}->{on}.

=cut

=head2 $gb->tRNA()

         Returns the array of all feature IDs of tRNAs.

=cut

=head2 $gb->rRNA()

         Returns the array of all feature IDs of rRNAs.

=cut

=head2 $gb->gene()

         Returns the array of all feature IDs of genes.

=cut

=head2 $gb->next_feature()

         Given a feature ID, returns the ID of the next feature.
         Second argument can be used to specify the type of the 
         next feature.

         eg. $gb->next_feature(FEATURE1234); # returns 'FEATURE1235'
             $gb->next_feature(FEATURE1234, 'tRNA'); 
             # returns next feature ID whose type is 'tRNA'

=cut

=head2 $gb->next_cds()

         Given a feature ID, returns the ID of the next cds.
         This is same as $gb->next_feature($featureID, 'CDS');

=cut

=head2 $gb->previous_feature()

         Given a feature ID, returns the ID of the previous feature.
         Second argument can be used to specify the type of the 
         next feature.

         eg. $gb->previous_feature(FEATURE1234); # returns 'FEATURE1233'
             $gb->previous_feature(FEATURE1234, 'tRNA'); 
             # returns previous feature ID whose type is 'tRNA'

=cut

=head2 $gb->previous_cds()

         Given a feature ID, returns the ID of the previous cds.
         This is same as $gb->previous_feature($featureID, 'CDS');

=cut




=head2 $gb->startcodon()

         Given a CDS ID, returns the start codon.

         eg. $gb->startcodon("FEATURE$i"); # returns 'atg'

=cut

=head2 $gb->stopcodon()

         Given a CDS ID, returns the stop codon.

         eg. $gb->stopcodon("FEATURE$i"); # returns 'tag'

=cut

=head2 $gb->before_startcodon()

         Given a CDS ID and length, returns the sequence upstream of 
         start codon.

         eg. $gb->before_startcodon('CDS1', 100); 
             # returns 100 bp sequence upstream of the start codon of 'CDS1'.

=cut

=head2 $gb->after_startcodon()

         Given a CDS ID and length, returns the sequence downstream of 
         start codon.

         eg. $gb->after_startcodon('CDS1', 100); 
             # returns 100 bp sequence downstream of the start codon of 'CDS1'.

=cut

=head2 $gb->before_stopcodon()

         Given a CDS ID and length, returns the sequence upstream of 
         stop codon.

         eg. $gb->before_stopcodon('CDS1', 100); 
             # returns 100 bp sequence upstream of the stop codon of 'CDS1'.

=cut

=head2 $gb->after_stopcodon()

         Given a CDS ID and length, returns the sequence downstream of 
         stop codon.

         eg. $gb->after_stopcodon('CDS1', 100); 
             # returns 100 bp sequence downstream of the stop codon of 'CDS1'.

=cut

=head2 $gb->get_exon()

         Given a CDS ID, returns the exon sequence.
         'complement' is properly parsed, and introns are spliced out.

         eg. $gb->get_exon('CDS1'); returns the 'CDS1' exon.

=cut


=head2 $gb->get_intron()

         Given a CDS ID, returns the intron sequences as array of 
         sequences.

         eg. $gb->get_intron('CDS1'); 
             # returns ($1st_intron, $2nd_intron,..)

=cut

=head2 $gb->pos2feature()

         Given a GenBank position (sequence starting from position 1) 
         returns the feature ID (ex. FEATURE123) of the feature at
         the given position. If multiple features exist for the given
         position, the first feature to appear is returned. Returns 
         NULL if no feature exists.

=cut

=head2 $gb->pos2gene()

         Given a GenBank position (sequence starting from position 1) 
         returns the feature ID (ex. FEATURE123) of the gene at
         the given position. If multiple genes exists for the given
         position, the first gene to appear is returned. Returns 
         NULL if no gene exists.

=cut

=head2 $gb->gene2id()

         Given a GenBank gene name, returns the feature ID (ex. FEATURE123). 
         Returns NULL if no gene exists.

=cut

=head2 $gb->next_locus()

         Reads the next locus.
         the G instance is then updated.

         eg. 
           do{
  
           }while($gb->next_locus());
           #  Enables multiple loci analysis.        

=cut

=head2 $gb->clone()

         Returns cloned G instance, which is a new G instance with
         identical data. 

=cut

=head2 $gb->DESTROY()

         Destroys the G instance.

=cut

=head2 $gb->del_key()

         Given a object, deletes it from the G instance structure
         eg. $gb->del_key('FEATURE1'); # deletes 'FEATURE1' hash

=cut


=head1 AUTHOR

Kazuharu Arakawa, gaou@sfc.keio.ac.jp

=cut



